package edu.nju.controller;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.nju.entities.*;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import edu.nju.service.ExtraService;
import org.springframework.web.multipart.MultipartFile;

import java.net.URL;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URLConnection;

@Controller
@RequestMapping(value = "/extra")
@CrossOrigin(origins = "*", maxAge = 3600, allowCredentials = "true")
public class ExtraController {
	@Autowired
	ExtraService extraService;
	
	//上传测试报告
	@RequestMapping(value = "/uploadReport", method = RequestMethod.POST)
	@ResponseBody
	public void uploadReport(String case_id, String task_id, String case_take_id, String worker_id, String name, String device_model,
			String device_brand, String device_os, String script_location, String report_location, 
			String log_location, HttpServletResponse response) {
		try {
			String id = extraService.saveReport(case_id, task_id, case_take_id, worker_id, name, device_model, device_brand, device_os, script_location, report_location, log_location);
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			result.put("id", id);
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	//上传测试报告
	@RequestMapping(value = "/updateReport", method = RequestMethod.POST)
	@ResponseBody
	public void updateReport(String report_id, String case_id, String task_id, String case_take_id, String worker_id, String name, String device_model,
							 String device_brand, String device_os, String script_location, String report_location,
							 String log_location, HttpServletResponse response) {
		try {
			boolean bool = extraService.updateReport(report_id, case_id, task_id, case_take_id, worker_id, name,
					device_model, device_brand, device_os, script_location, report_location, log_location);
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			if(bool) { result.put("status", 200); }
			else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//获取测试报告
	@RequestMapping(value = "/getReport")
	@ResponseBody
	public void getReport(String report_id, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			Report report = extraService.getReport(report_id);
			if(report != null) { 
				result.put("status", 200);
				result.put("result", new JSONObject(report));
			}
			else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//使用worker_id获取测试报告
	@RequestMapping(value = "/findByWorker")
	@ResponseBody
	public void findByWorker(String case_take_id, String worker_id, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			Report report = extraService.findByWorker(case_take_id, worker_id);
			if(report != null) { 
				result.put("status", 200);
				result.put("result", new JSONObject(report));
			}
			else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	//获取测试用例列表
	@RequestMapping(value = "/getCaseList")
	@ResponseBody
	public void getCaseList(String report_id, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<TestCase> caseList = extraService.getCaseList(report_id);
			if(caseList != null) { 
				result.put("status", 200);
				result.put("result", new JSONArray(caseList));
			}
			else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//获取测试Bug列表
	@RequestMapping(value = "/getBugList")
	@ResponseBody
	public void getBugList(String report_id, String case_take_id, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			List<Bug> bugList = extraService.getBugList(report_id, case_take_id);
			if(bugList != null && bugList.size() > 0) { 
				result.put("status", 200);
				result.put("result", new JSONArray(bugList));
			}
			else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 47.99.140.117:9001/Bug/api/extra/uploadExam
	 *
	 * @param file
	 *            文件流
	 * @param case_id
	 * 			  测试目标id
	 * @param file_name
	 *            上传excel的文件名
	 * @param paper_type
	 * 			  试卷类型，即试卷的显示矩阵
	 * @param test_type
	 * 			  测试分类，如工具
	 * @param description
	 * 			  测试说明
	 * @param app_name
	 * 			  测试目标的名称
	 * @return JSONArray
	 */
	@RequestMapping(value = "/uploadExam", method = RequestMethod.POST)
	@ResponseBody
	public String uploadExam(@RequestParam("file") MultipartFile file, String file_name, String paper_type,
							     String case_id, String test_type, String description, String app_name) {
		try {
			File dest = new File("/Users/hannatao/Downloads/" + file_name);
			if(!dest.getParentFile().exists()) { dest.getParentFile().mkdirs(); }
			if(!file.isEmpty()) { file.transferTo(dest); }
			String json = extraService.saveExam(case_id, dest.getPath(), app_name, paper_type, test_type, description);
			return json;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}

	/**
	 *
	 * @param file 文件oss的url
	 * @param file_name
	 * @param paper_type
	 * @param case_id
	 * @param test_type
	 * @param description
	 * @param app_name
	 * @return
	 */
	@RequestMapping(value = "/uploadExamUrl", method = RequestMethod.POST)
	@ResponseBody
	public String uploadExamUrl(String file, String file_name, String paper_type,
								String case_id, String test_type, String description, String app_name) {
		try {

			File dest = new File("/Users/hannatao/Downloads/" + file_name);
			if(!dest.getParentFile().exists()) { dest.getParentFile().mkdirs(); }
			//从oss下载文件
			// 统一资源
			URL url = new URL(file);
			// 连接类的父类，抽象类
			URLConnection urlConnection = url.openConnection();
			// http的连接类
			HttpURLConnection httpURLConnection = (HttpURLConnection) urlConnection;
			//设置超时
			httpURLConnection.setConnectTimeout(1000*5);
			//设置请求方式，默认是GET
//          httpURLConnection.setRequestMethod("GET");
			// 设置字符编码
			httpURLConnection.setRequestProperty("Charset", "UTF-8");
			// 打开到此 URL引用的资源的通信链接（如果尚未建立这样的连接）。
			httpURLConnection.connect();
			// 文件大小
			int fileLength = httpURLConnection.getContentLength();

			// 建立链接从请求中获取数据
			URLConnection con = url.openConnection();
			BufferedInputStream bin = new BufferedInputStream(httpURLConnection.getInputStream());
			// 指定文件名称(有需求可以自定义)
			// 指定存放位置(有需求可以自定义)


			OutputStream out = new FileOutputStream(dest);
			int size = 0;
			int len = 0;
			byte[] buf = new byte[2048];
			while ((size = bin.read(buf)) != -1) {
				len += size;
				out.write(buf, 0, size);
			}
			// 关闭资源
			bin.close();
			out.close();
			String json = extraService.saveExam(case_id, dest.getPath(), app_name, paper_type, test_type, description);
			return json;
		} catch (IOException e) {
			e.printStackTrace();
			return "";
		}
	}

//	@RequestMapping(value = "/uploadExamUrl", method = RequestMethod.POST)
//	@ResponseBody
//	public String uploadExamUrl1(String file, String file_name, String paper_type,
//								String case_id, String test_type, String description, String app_name,
//								String if_test_case,String if_bug) {
//		try {
//
//			File dest = new File("/Users/hannatao/Downloads/" + file_name);
//			if(!dest.getParentFile().exists()) { dest.getParentFile().mkdirs(); }
//			//从oss下载文件
//			// 统一资源
//			URL url = new URL(file);
//			// 连接类的父类，抽象类
//			URLConnection urlConnection = url.openConnection();
//			// http的连接类
//			HttpURLConnection httpURLConnection = (HttpURLConnection) urlConnection;
//			//设置超时
//			httpURLConnection.setConnectTimeout(1000*5);
//			//设置请求方式，默认是GET
////          httpURLConnection.setRequestMethod("GET");
//			// 设置字符编码
//			httpURLConnection.setRequestProperty("Charset", "UTF-8");
//			// 打开到此 URL引用的资源的通信链接（如果尚未建立这样的连接）。
//			httpURLConnection.connect();
//			// 文件大小
//			int fileLength = httpURLConnection.getContentLength();
//
//			// 建立链接从请求中获取数据
//			URLConnection con = url.openConnection();
//			BufferedInputStream bin = new BufferedInputStream(httpURLConnection.getInputStream());
//			// 指定文件名称(有需求可以自定义)
//			// 指定存放位置(有需求可以自定义)
//
//
//			OutputStream out = new FileOutputStream(dest);
//			int size = 0;
//			int len = 0;
//			byte[] buf = new byte[2048];
//			while ((size = bin.read(buf)) != -1) {
//				len += size;
//				out.write(buf, 0, size);
//			}
//			// 关闭资源
//			bin.close();
//			out.close();
//			String json = extraService.saveExam1(case_id, dest.getPath(), app_name, paper_type, test_type, description,if_test_case,if_bug);
//			return json;
//		} catch (IOException e) {
//			e.printStackTrace();
//			return "";
//		}
//	}

	/**
	 * 47.99.140.117:9001/Bug/api/extra/getExamList
	 *
	 * @return 200 成功； 500 失败
	 */
	@RequestMapping(value = "/getExamList", method = RequestMethod.GET)
	@ResponseBody
	public void getExamList(HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			result.put("data", extraService.getExamList());
			result.put("status", 200);
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//获取题目信息
	@RequestMapping(value = "/getExam")
	@ResponseBody
	public void getExam(String id, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			Exam exam = extraService.getExam(id);
			if(exam != null) { 
				result.put("status", 200);
				result.put("result", new JSONObject(exam));
			}
			else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//上传测试用例
	@RequestMapping(value = "/uploadTestCase", method = RequestMethod.POST)
	@ResponseBody
	public void uploadTestCase(String report_id, String name, String front, String behind, 
			String description,String if_execute,String if_bug, HttpServletResponse response) {
		try {

			System.out.println(if_execute);
			System.out.println(if_bug);
			String id = extraService.saveTestCase(report_id, name, front, behind, description);
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			result.put("id", id);
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//获取测试用例
	@RequestMapping(value = "/getTestCase")
	@ResponseBody
	public void getTestCase(String id, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			TestCase testCase = extraService.getTestCase(id);
			if(testCase != null) { 
				result.put("status", 200);
				result.put("result", new JSONObject(testCase));
			}
			else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//更新测试用例
	@RequestMapping(value = "/updateTestCase", method = RequestMethod.POST)
	@ResponseBody
	public void updateTestCase(String id, String report_id, String name, String front, String behind, 
			String description, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			if(extraService.updateTestCase(id, report_id, name, front, behind, description)) {
				result.put("status", 200);
			} else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//获取页面json
	@RequestMapping(value = "/getPageJson")
	@ResponseBody
	public void getPageJson(String caseId, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			Exam exam = extraService.getExam(caseId);
			if(exam != null) { 
				result.put("status", 200);
				result.put("result", exam.getJson());
			}
			else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}


	@RequestMapping(value = "/updateTask", method = RequestMethod.POST)
	@ResponseBody
	public void updateTask(String id,double writeMins, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			if(extraService.updateTask(id, writeMins)) {
				result.put("status", 200);
			}
			else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@RequestMapping(value = "/getTask")
	@ResponseBody
	public void getTask(String id, HttpServletResponse response) {
		try {
			PrintWriter out = response.getWriter();
			JSONObject result = new JSONObject();
			Task task=extraService.saveTask(id);
			if(task!=null) {
				result.put("status", 200);
				result.put("result",new JSONObject(task));
			}
			else { result.put("status", 500); }
			out.print(result);
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@RequestMapping(value = "/reportsToReviewPaper")
	@ResponseBody
	public String getTask(String case_take_id) {
		String uploadUrl = extraService.reportsToReviewPaper(case_take_id);
		return uploadUrl;
	}

}
