package edu.nju.controller;

import edu.nju.entities.*;
import edu.nju.model.ReviewWorkerVO;
import edu.nju.service.ReviewService;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

@Controller
@RequestMapping(value = "/review")
@CrossOrigin(origins = "*", maxAge = 3600, allowCredentials = "true")
public class ReviewController {

    @Autowired
    ReviewService iservice;

    @RequestMapping(value = "/getReportsByWorkerJob")
    @ResponseBody
    public void getReportListByWorkerJob(String job_id, String worker_id, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            List<ReviewReport> reports = iservice.getReportsByWorkerJob(worker_id, job_id);
            out.print(new JSONArray(reports));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getItemsByReport")
    @ResponseBody
    public void getItemsByReport(String reportId, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            List<ReviewItem> items = iservice.getItemsByReport(reportId);
            out.print(new JSONArray(items));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    @RequestMapping(value = "/getAnswerByWorkerJobReportItem")
    @ResponseBody
    public void getAnswerByWorkerJobReportItem(String jobId,String reportId,String itemId,String workerId, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            ReviewAnswer answers = iservice.getAnswerByWorkerJobReportItem(jobId,reportId,workerId,itemId);
            out.print(new JSONObject(answers));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/saveAnswer", method = RequestMethod.POST)
    @ResponseBody
    public void saveAnswer(String answerId, @RequestParam("answers")List<String>answers, @RequestParam("attachmentLocation") List<String> attachmentLocation, HttpServletResponse response){
        JSONObject result = new JSONObject();
        if(iservice.saveAnswer(answerId,answers,attachmentLocation)){
            result.put("status", "200");
        } else {
            result.put("status", "500");
        }
        try {
            PrintWriter out = response.getWriter();
            out.print(result);
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getWorkersByJob")
    @ResponseBody
    public void getWorkers(String jobId, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            List<ReviewWorkerVO> reviewWorkerVOs = iservice.getWorkersByJob(jobId);
            out.print(new JSONArray(reviewWorkerVOs));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getReportsByJob")
    @ResponseBody
    public void getReportsByJob(String jobId, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            List<ReviewReport> reviewReports = iservice.getReportsByJob(jobId);
            out.print(new JSONArray(reviewReports));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getCorrectAnswerByReportItemJob")
    @ResponseBody
    public void getCorrectAnswerByReportItem(String reportId,String itemId,String jobId, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            ReviewCorrectAnswer reviewCorrectAnswer=iservice.getCorrectAnswerByReportItem(reportId, itemId, jobId);
            out.print(new JSONObject(reviewCorrectAnswer));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/saveCorrectAnswer", method = RequestMethod.POST)
    @ResponseBody
    public void updateCorrectAnswer(String answerId,@RequestParam("answer")List<String> answer,@RequestParam("attachment_location") List<String> file_url, String grade, HttpServletResponse response) {
        JSONObject result = new JSONObject();
//        System.out.println(answerId);
//        System.out.println(answer);
//        System.out.println(file_url);
//        System.out.println(grade);
        if (iservice.updateCorrectAnswer(answerId, answer, file_url, grade)) {
            result.put("status", "200");
        } else {
            result.put("status", "500");
        }
        try {
            PrintWriter out = response.getWriter();
            out.print(result);
            out.flush();
            out.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getItemAnswers")
    @ResponseBody
    public void getItemReportJobAnswers(String itemId, String reportId,String jobId, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            List<ReviewAnswer> reviewAnswers = iservice.getItemReportJobAnswers(itemId,reportId,jobId);
            out.print(new JSONArray(reviewAnswers));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/setGrade", method = RequestMethod.POST)
    @ResponseBody
    public void setGrade(String jobId,String itemId, String reportId, String workerId,String grade, HttpServletResponse response){
        JSONObject result = new JSONObject();
        if(iservice.setGrade(jobId,itemId,reportId,workerId,grade)){
            result.put("status", "200");
        } else {
            result.put("status", "500");
        }
        try {
            PrintWriter out = response.getWriter();
            out.print(result);
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }


}
