package edu.nju.controller;

import edu.nju.entities.ReviewPaper;
import edu.nju.entities.ReviewPaperJson;
import edu.nju.entities.ReviewReport;
import edu.nju.entities.ReviewWorker;
import edu.nju.model.CrowdReviewReportDTO;
import edu.nju.model.CrowdReviewReportVO;
import edu.nju.service.ReviewPaperService;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

@Controller
@RequestMapping(value = "/paper")
@CrossOrigin(origins = "*", maxAge = 3600, allowCredentials = "true")
public class ReviewPaperController {

    @Autowired
    ReviewPaperService reviewPaperService;

    @RequestMapping(value = "/uploadReport", method = RequestMethod.POST)
    @ResponseBody
    public  CrowdReviewReportVO uploadReport(@RequestBody CrowdReviewReportDTO crowdReviewReportDTO){
        CrowdReviewReportVO result=reviewPaperService.uploadReport(crowdReviewReportDTO);
        return result;
    }

    @RequestMapping(value = "/uploadPaper", method = RequestMethod.POST)
    @ResponseBody
    public void uploadPaper(String paperJson, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            JSONObject result = reviewPaperService.uploadPaper(paperJson);
            out.print(result);
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getPaperJson")
    @ResponseBody
    public void getPaperJson(String paperId, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            ReviewPaperJson paperJson = reviewPaperService.getReviewPaperJson(paperId);
            out.print(new JSONObject(paperJson));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getPaper")
    @ResponseBody
    public void getPaperById(String paperId, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            ReviewPaper paper = reviewPaperService.getPaper(paperId);
            out.print(new JSONObject(paper));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getReportsByPaper")
    @ResponseBody
    public void getReportsByPaper(String paperId, HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            List<ReviewReport> reports = reviewPaperService.getReportsByPaperId(paperId);
            out.print(new JSONArray(reports));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getPapers")
    @ResponseBody
    public void getPapers(HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            List<ReviewPaper> papers = reviewPaperService.getPapers();
            out.print(new JSONArray(papers));
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/delPaper")
    @ResponseBody
    public void deletePaper(String paperId, HttpServletResponse response){
        try {
            JSONObject result = new JSONObject();
            if(reviewPaperService.delPaper(paperId)){
                result.put("status", "200");
            } else {
                result.put("status", "500");
            }
            PrintWriter out = response.getWriter();
            out.print(result);
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/updatePaper")
    @ResponseBody
    public void updatePaper(String paperId, String paperJson,HttpServletResponse response){
        try {
            PrintWriter out = response.getWriter();
            JSONObject result = reviewPaperService.updatePaper(paperId,paperJson);
            out.print(result);
            out.flush();
            out.close();
        }catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @RequestMapping(value = "/getPaperByWorker")
    @ResponseBody
    public String getPaperByWorker(@RequestParam String examId,@RequestParam String caseId, @RequestParam String workerIndex ){
        String url=reviewPaperService.getWorkerPaperUrl(workerIndex,examId,caseId);
        return url;
    }


}
