package edu.nju.controller.interceptor;

import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author: Diors.Po
 * @Email: 171256175@qq.com
 * @date 2019-09-29 14:15
 */
@Slf4j
@Component
public class AuthCheckInterceptor extends HandlerInterceptorAdapter {
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if (hasAuthRequired(handler)){
            log.info("需要认证的接口访问行为");
            if (request.getSession().getAttribute("userInfo") == null)
                throw new HttpClientErrorException(HttpStatus.UNAUTHORIZED);
            else
                log.info("用户已登录, userInfo:"+request.getSession().getAttribute("userInfo"));
        }
        return true;
    }

    private boolean hasAuthRequired(Object handler) {
        if(handler instanceof HandlerMethod) {
            HandlerMethod handlerMethod = (HandlerMethod) handler;
            AuthRequired authRequired = handlerMethod.getMethod().getAnnotation(AuthRequired.class);
            if(authRequired==null){
                authRequired = handlerMethod.getBeanType().getAnnotation(AuthRequired.class);
            }
            return !(authRequired == null);
        }
        return false;
    }
}
