package edu.nju.dao;

import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;
import edu.nju.entities.ReviewAnswer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Repository;

import java.util.*;

@Repository
public class ReviewAnswerDao {

    @Autowired
    private MongoOperations mongoOperations;

    public ReviewAnswer findAnswerByWorkerJobReportItem(String jobId, String reportId, String workerId, String itemId){
        Query query = new Query();
        query.addCriteria(Criteria.where("job_id").is(jobId).and("report_id").is(reportId).and("worker_id").is(workerId).and("item_id").is(itemId));
        List<ReviewAnswer> answers= mongoOperations.find(query, ReviewAnswer.class);
        if(answers==null||answers.size()==0){
            return null;
        }
        return answers.get(0);
    }

    //存在则更新，不存在则插入
    public String save(ReviewAnswer answer){
        mongoOperations.save(answer);
        return answer.getId();
    }

    public ReviewAnswer getAnswerById(String id){
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is(id));
        List<ReviewAnswer> list = mongoOperations.find(query, ReviewAnswer.class);
        if(list == null||list.size() == 0) {return null;}
        return list.get(0);
    }

    public List<ReviewAnswer>getItemReportJobAnswers(String itemId,String reportId,String jobId){
        Query query = new Query();
        query.addCriteria(Criteria.where("item_id").is(itemId).and("report_id").is(reportId).and("job_id").is(jobId));
        List<ReviewAnswer> answers= mongoOperations.find(query, ReviewAnswer.class);
        return answers;
    }

    public List<ReviewAnswer>getAnswersByItem(String itemId){
        Query query = new Query();
        query.addCriteria(Criteria.where("item_id").is(itemId));
        List<ReviewAnswer> answers= mongoOperations.find(query, ReviewAnswer.class);
        return answers;
    }

    public List<ReviewAnswer> getAnswersByJob(String jobId){
//        String[] field = {"report_id","item_id"}; 		//分组的依据，根据那些参数进行分组
//        Aggregation aggregation1 = Aggregation.newAggregation(
//                Aggregation.match(Criteria.where("job_id").is(jobId)),	//条件筛选    可以跟好几个条件
//                Aggregation.group(field).count().as("sum"),		//分组统计收藏量    ，as 起别名
//                Aggregation.sort(new Sort(Sort.Direction.DESC, "sum"))  //按照收藏量倒叙排序
//        );
//        //执行
//        AggregationResults<BasicDBObject> outputTypeCount1 = mongoOperations.aggregate(aggregation1, "collect", BasicDBObject.class);
//        //迭代器进行迭代
//        Map<String,String>result=new HashMap<>();
//        for (Iterator<BasicDBObject> iterator = outputTypeCount1.iterator(); iterator.hasNext(); ) {
//            DBObject obj = iterator.next();
//            String sum = obj.get("sum").toString();
//            String itemId = obj.get("sum").toString();
//            result.put(itemId,sum);
//        }
//        return result;
        Query query = new Query();
        query.addCriteria(Criteria.where("job_id").is(jobId));
        List<ReviewAnswer> answers= mongoOperations.find(query, ReviewAnswer.class);
        return answers;
    }

    public ReviewAnswer findLastAnswerByWorkerJob(String jobId, String workerId){
        Query query = new Query();
        query.addCriteria(Criteria.where("job_id").is(jobId).and("worker_id").is(workerId));
        List<ReviewAnswer> answers= mongoOperations.find(query, ReviewAnswer.class);
        if(answers==null||answers.size()==0){
            return null;
        }
        return answers.get(answers.size()-1);
    }

//    public List<ReviewAnswer>getReportJobWorkerAnswers(String reportId,String jobId,String workerId){
//        Query query = new Query();
//        query.addCriteria(Criteria.where("worker_id").is(workerId).and("report_id").is(reportId).and("job_id").is(jobId));
//        List<ReviewAnswer> answers= mongoOperations.find(query, ReviewAnswer.class);
//        return answers;
//    }

}
