package edu.nju.service;

import edu.nju.dao.*;
import edu.nju.entities.*;
import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import edu.nju.util.HTTP;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
@Service
public class NodeService {

    @Autowired
    AnswerNodeDao answerNodeDao;

    @Autowired
    BasicNodeDao basicNodeDao;

    @Autowired
    ReviewItemDao reviewItemDao;

    @Autowired
    ReviewJobDao reviewJobDao;

    @Autowired
    ReviewPaperDao reviewPaperDao;

    @Autowired
    ReviewReportDao reviewReportDao;

    @Autowired
    ReviewCorrectAnswerDao reviewCorrectAnswerDao;

    @Autowired
    ReviewAnswerDao reviewAnswerDao;

    @Autowired
    ReviewWorkerDao reviewWorkerDao;


    Logger logger= LoggerFactory.getLogger(RecommendService.class);


    public BasicNode generateBasicNode(String jobId,String teacherId){
        ReviewJob reviewJob=reviewJobDao.findJob(jobId);
        ReviewPaper reviewPaper=reviewPaperDao.findPaper(reviewJob.getPaper_id());
        List<ReviewReport> reviewReportList=reviewReportDao.getReportsByPaperId(reviewPaper.getId());
        //构建元node
        JSONObject nodeJson = new JSONObject();
        JSONArray categoryArray=new JSONArray();
        JSONObject categoryObject=new JSONObject();
        categoryObject.put("name","CrowdReview");
        categoryArray.put(categoryObject);
        nodeJson.put("categories",categoryArray);
        JSONArray nodesArray=new JSONArray();
        for(ReviewReport reviewReport:reviewReportList){
            String reportId=reviewReport.getId();
            List<ReviewItem> reviewItemList=reviewItemDao.findItemsByReport(reportId);
            for(ReviewItem reviewItem:reviewItemList){
                String itemId=reviewItem.getId();
                ReviewCorrectAnswer correctAnswer=reviewCorrectAnswerDao.findCorrectAnswerByReportItem(reportId,itemId,jobId);
                if(reviewItem.getType().equals("Multiple")){
                    List<String> answers=correctAnswer.getCorrect_answer();
                    int num=0;
                    for(int i=0;i<answers.size();i++){
                        if(answers.get(i).equals("1")){
                            num++;
                        }
                    }
                    if(num!=0){
                        for(int i=0;i<answers.size();i++){
                            String answer=answers.get(i);
                            if(answer.equals("1")){
                                JSONObject node=new JSONObject();
                                node.put("name", jobId + "/" + reportId + "/" + itemId+"/"+i);
                                node.put("location", jobId + "/" + reportId + "/" + itemId+"/"+i);
                                node.put("description", reviewItem.getType());
                                node.put("category", "CrowdReview");
                                node.put("weight", String.valueOf(Integer.parseInt(correctAnswer.getGrade())/num));
                                nodesArray.put(node);
                            }
                        }
                    }
                }else {
                    JSONObject node=new JSONObject();
                    node.put("name", jobId + "/" + reportId + "/" + itemId);
                    node.put("location", jobId + "/" + reportId + "/" + itemId);
                    node.put("description", reviewItem.getType());
                    node.put("category", "CrowdReview");
                    node.put("weight", correctAnswer.getGrade());
                    nodesArray.put(node);
                }
            }
        }
        nodeJson.put("nodes",nodesArray);
        //查询是否已有之前的basicNode
        BasicNode basicNode=basicNodeDao.findByJobTeacher(jobId,teacherId);
        //新建
        if(basicNode==null){
            basicNode=new BasicNode(jobId,teacherId,nodeJson.toString());
            basicNodeDao.save(basicNode);
        }
        //更新
        else{
            basicNode.setNodeJson(nodeJson.toString());
            basicNodeDao.save(basicNode);
        }
        System.out.println(nodeJson);
        return basicNode;
    }


    public AnswerNode generateCatchNode(String jobId,String workerId){
//        System.out.println("job"+jobId);
//        System.out.println("worker"+workerId);
        ReviewJob reviewJob=reviewJobDao.findJob(jobId);
        ReviewPaper reviewPaper=reviewPaperDao.findPaper(reviewJob.getPaper_id());
        List<ReviewReport> reviewReportList=reviewReportDao.getReportsByPaperId(reviewPaper.getId());
        //构建catchNode
        JSONArray catchNodeArray = new JSONArray();
        for(ReviewReport reviewReport:reviewReportList){
            String reportId=reviewReport.getId();
            List<ReviewItem> reviewItemList=reviewItemDao.findItemsByReport(reportId);
            for(ReviewItem reviewItem:reviewItemList){
                String itemId=reviewItem.getId();
                //只要进入过答题界面，所有reviewAnswer就会被创建
                ReviewAnswer reviewAnswer=reviewAnswerDao.findAnswerByWorkerJobReportItem(jobId,reportId,workerId,itemId);
                ReviewCorrectAnswer reviewCorrectAnswer=reviewCorrectAnswerDao.findCorrectAnswerByReportItem(reportId,itemId,jobId);
                //多选题
                switch (reviewItem.getType()) {
                    case "Multiple":
                        generateCatchNodeMultiple(catchNodeArray, reviewAnswer, reviewCorrectAnswer, jobId, reportId, itemId);
                        break;
                    //单选题
                    case "Single":
                        generateCatchNodeSingle(catchNodeArray, reviewAnswer, reviewCorrectAnswer, jobId, reportId, itemId);
                        break;
                    //描述题和文件题，默认只要有作答就得分
                    case "Description":
                        generateCatchNodeDescription(catchNodeArray, reviewAnswer, reviewCorrectAnswer, jobId, reportId, itemId);
                        break;
                    case "File":
                        generateCatchNodeFile(catchNodeArray, reviewAnswer, reviewCorrectAnswer, jobId, reportId, itemId);
                        break;
                }
            }
        }

        System.out.println(catchNodeArray);

        AnswerNode answerNode=answerNodeDao.findByJobWorker(jobId,workerId);
        if(answerNode==null){
            answerNode=new AnswerNode(jobId,workerId,catchNodeArray.toString(),"0");
        }else{
            answerNode.setCatchNodeJson(catchNodeArray.toString());
        }
        answerNodeDao.save(answerNode);
//        test();
        return answerNode;
    }

    private void generateCatchNodeMultiple(JSONArray catchNodeArray,ReviewAnswer reviewAnswer,ReviewCorrectAnswer reviewCorrectAnswer,String jobId,String reportId,String itemId){
        if(reviewAnswer!=null) {
            List<String> answers = reviewAnswer.getAnswers();
            List<String> correctAnswers = reviewCorrectAnswer.getCorrect_answer();
            int num = 0;
            //共有几个正确选项
            for (int i = 0; i < correctAnswers.size(); i++) {
                if (correctAnswers.get(i).equals("1")) {
                    num++;
                }
            }
            if (num != 0) {
                List<JSONObject> multipleNodeList = new ArrayList<>();
                boolean chooseWrong = false;
                //是否作答该题
                if (answers.size() != correctAnswers.size()) {
                    for (int i = 0; i < correctAnswers.size(); i++) {
                        JSONObject node = new JSONObject();
                        node.put("category", "CrowdReview");
                        node.put("ifCatch", false);
                        node.put("nodeName", jobId + "/" + reportId + "/" + itemId + "/" + i);
                        multipleNodeList.add(node);
                    }
                    chooseWrong = true;
                } else {
                    for (int i = 0; i < correctAnswers.size(); i++) {
                        String correctAnswer = correctAnswers.get(i);
                        String answer = answers.get(i);
                        JSONObject node = new JSONObject();
                        if (correctAnswer.equals("1")) {
                            if (answer.equals("1")) {
                                node.put("category", "CrowdReview");
                                node.put("ifCatch", true);
                                node.put("nodeName", jobId + "/" + reportId + "/" + itemId + "/" + i);
                            } else {
                                node.put("category", "CrowdReview");
                                node.put("ifCatch", false);
                                node.put("nodeName", jobId + "/" + reportId + "/" + itemId + "/" + i);
                            }
                            multipleNodeList.add(node);
                        } else if (correctAnswer.equals("")) {
                            if (answer.equals("1")) {
                                chooseWrong = true;
                                break;
                            }
                        }
                    }
                }
                //有一个选错，则多选题不得分
                if (chooseWrong) {
                    for (JSONObject node : multipleNodeList) {
                        node.put("ifCatch", false);
                        catchNodeArray.put(node);
                    }
                }
                //否则按选中的个数得分
                else {
                    for (JSONObject node : multipleNodeList) {
                        catchNodeArray.put(node);
                    }
                }
            }
        }
        //该工人未进入答题界面
        else{
            List<String> correctAnswers = reviewCorrectAnswer.getCorrect_answer();
            int num = 0;
            //共有几个正确选项
            for (int i = 0; i < correctAnswers.size(); i++) {
                if (correctAnswers.get(i).equals("1")) {
                    num++;
                }
            }
            if (num != 0) {
                List<JSONObject> multipleNodeList = new ArrayList<>();
                for (int i = 0; i < correctAnswers.size(); i++) {
                    JSONObject node = new JSONObject();
                    node.put("category", "CrowdReview");
                    node.put("ifCatch", false);
                    node.put("nodeName", jobId + "/" + reportId + "/" + itemId + "/" + i);
                    multipleNodeList.add(node);
                }
                for (JSONObject node : multipleNodeList) {
                    catchNodeArray.put(node);
                }
            }

        }

    }


    private void generateCatchNodeSingle(JSONArray catchNodeArray,ReviewAnswer reviewAnswer,ReviewCorrectAnswer reviewCorrectAnswer,String jobId,String reportId,String itemId){
        if(reviewAnswer!=null) {
            List<String> answers = reviewAnswer.getAnswers();
            List<String> correctAnswers = reviewCorrectAnswer.getCorrect_answer();
            boolean chooseWrong = false;
            if (answers.size() != correctAnswers.size()) {
                chooseWrong = true;
            } else {
                for (int i = 0; i < correctAnswers.size(); i++) {
                    String correctAnswer = correctAnswers.get(i);
                    String answer = answers.get(i);
                    if (!answer.equals(correctAnswer)) {
                        chooseWrong = true;
                        break;
                    }
                }
            }
            JSONObject node = new JSONObject();
            node.put("category", "CrowdReview");
            node.put("nodeName", jobId + "/" + reportId + "/" + itemId);
            node.put("ifCatch", !chooseWrong);
            catchNodeArray.put(node);
        }else{
            JSONObject node = new JSONObject();
            node.put("category", "CrowdReview");
            node.put("nodeName", jobId + "/" + reportId + "/" + itemId);
            node.put("ifCatch", false);
            catchNodeArray.put(node);
        }
    }

    private void generateCatchNodeDescription(JSONArray catchNodeArray,ReviewAnswer reviewAnswer,ReviewCorrectAnswer reviewCorrectAnswer,String jobId,String reportId,String itemId){
        if(reviewAnswer!=null) {
            List<String> answers = reviewAnswer.getAnswers();
            boolean hasContent = false;
            for (String answer : answers) {
                if (!answer.equals("")) {
                    hasContent = true;
                    break;
                }
            }
            JSONObject node = new JSONObject();
            node.put("category", "CrowdReview");
            node.put("nodeName", jobId + "/" + reportId + "/" + itemId);
            node.put("ifCatch", hasContent);
            catchNodeArray.put(node);
        }else{
            JSONObject node = new JSONObject();
            node.put("category", "CrowdReview");
            node.put("nodeName", jobId + "/" + reportId + "/" + itemId);
            node.put("ifCatch", false);
            catchNodeArray.put(node);
        }
    }

    private void generateCatchNodeFile(JSONArray catchNodeArray,ReviewAnswer reviewAnswer,ReviewCorrectAnswer reviewCorrectAnswer,String jobId,String reportId,String itemId){
        if(reviewAnswer!=null) {
            List<String> fileList = reviewAnswer.getFile_url();
            boolean hasContent = false;
            for (String file : fileList) {
                if (!file.equals("")) {
                    hasContent = true;
                    break;
                }
            }
            JSONObject node = new JSONObject();
            node.put("category", "CrowdReview");
            node.put("nodeName", jobId + "/" + reportId + "/" + itemId);
            node.put("ifCatch", hasContent);
            catchNodeArray.put(node);
        }else{
            JSONObject node = new JSONObject();
            node.put("category", "CrowdReview");
            node.put("nodeName", jobId + "/" + reportId + "/" + itemId);
            node.put("ifCatch", false);
            catchNodeArray.put(node);
        }
    }

    private void generateCatchNodeFalse(JSONArray catchNodeArray,String jobId,String reportId,String itemId){
        JSONObject node = new JSONObject();
        node.put("category", "CrowdReview");
        node.put("nodeName", jobId + "/" + reportId + "/" + itemId);
        node.put("ifCatch", false);
        catchNodeArray.put(node);
    }

    public BasicNode getBasicNode(String jobId,String teacherId){
        return basicNodeDao.findByJobTeacher(jobId,teacherId);
    }

    public AnswerNode getAnswerNode(String jobId,String workerId){
        return answerNodeDao.findByJobWorker(jobId,workerId);
    }

    public JSONObject uploadSingleCatchNodeToMoocTest(String jobId,String workerId){
        ReviewJob reviewJob=reviewJobDao.findJob(jobId);
        ReviewWorker reviewWorker=reviewWorkerDao.find(workerId);
        AnswerNode answerNode=generateCatchNode(jobId,workerId);
        JSONObject result=new JSONObject();
        String uploadId=getCatchNodeUploadId(reviewJob.getExam_id(),answerNode,reviewWorker.getOriginal_id());
        logger.info(String.valueOf(uploadId));
        logger.info("caseid"+reviewJob.getCase_id());
        if(uploadId!=null){
            //上传catchNode至主站
            String uploadResult=uploadCatchNode(reviewJob.getCase_id(),uploadId,answerNode);
            logger.info(String.valueOf(uploadResult));
            if(uploadResult==null){
                logger.info("uploadResult不对");
                result.put("status","500");
                result.put("cause","上传catchNode失败");
            }else{
                JSONObject uploadResultJson=new JSONObject(uploadResult);
                if(String.valueOf(uploadResultJson.get("code")).equals("20000")){
//                    logger.info((String) uploadResultJson.get("code"));
//                    logger.info("code对");
                    result.put("status","200");
                }else{
//                    logger.info((String) uploadResultJson.get("code"));
//                    logger.info("code不对");
                    result.put("status","500");
                    result.put("cause","上传catchNode失败");
                }
            }
        }else{
            result.put("status","500");
            result.put("cause","获取uploadId失败");
        }
//        test();
        logger.info(result.toString());
        return result;
    }

    public JSONObject uploadCatchNodeToMoocTest(String jobId){
        //todo 异步调用
         ReviewJob reviewJob=reviewJobDao.findJob(jobId);
         List<ReviewWorker> reviewWorkerList=reviewWorkerDao.getWorkersByJob(jobId);
         JSONObject result=new JSONObject();
         for(ReviewWorker reviewWorker:reviewWorkerList){
             AnswerNode answerNode=generateCatchNode(jobId,reviewWorker.getId());
             //获取上传学生成绩至主站的uploadId
             String uploadId=getCatchNodeUploadId(reviewJob.getExam_id(),answerNode,reviewWorker.getOriginal_id());
             if(uploadId!=null){
                 //上传catchNode至主站
                 String uploadResult=uploadCatchNode(reviewJob.getCase_id(),uploadId,answerNode);
                 if(uploadResult==null){
                     result.put("status","500");
                     result.put("cause","上传catchNode失败");
                 }else{
                     JSONObject uploadResultJson=new JSONObject(uploadResult);
                     if(uploadResultJson.get("code").equals("20000")){
                         result.put("status","200");
                     }else{
                         result.put("status","500");
                         result.put("cause","上传catchNode失败");
                     }
                 }
             }else{
                 result.put("status","500");
                 result.put("cause","获取uploadId失败");
             }

         }
         return result;
    }

    private String getCatchNodeUploadId(String taskId,AnswerNode answerNode,String workerOriginalId){
        String url = "http://www.mooctest.net/api/dev/plugin/submitAnswersSig" + "?" +
                "taskID=" + taskId + "&" +
                "workerID=" + workerOriginalId + "&" +
                "caseName=CrowdReview";
        String result = HTTP.sendGet(url, "");
        if(!result.equals("")) {
            JSONObject json = new JSONObject(result);
            if(json.has("data")){
                JSONObject data=json.getJSONObject("data");
                return String.valueOf(data.getInt("uploadId"));
            }
        }
        return null;
    }

    //向慕测主站上传catchNode
    private String uploadCatchNode(String caseId,String uploadId,AnswerNode answerNode){
        logger.info("开始上传至主站");
        logger.info(String.valueOf(uploadId));
        logger.info(caseId);
        logger.info(answerNode.getCatchNodeJson());
        StringBuilder result = new StringBuilder();
        String uploadTime=String.valueOf(System.currentTimeMillis());
        String source="CrowdReview";
        String url="http://www.mooctest.net/api/common/generalUploadCaughtNode/"+caseId+"/"+uploadTime+"/"+uploadId+"/"+source;
        try{
            URL realUrl = new URL(url);
            // 打开和URL之间的连接
            HttpURLConnection connection;
            connection = (HttpURLConnection)realUrl.openConnection();
            connection.setDoInput(true);
            connection.setDoOutput(true);
            // 设置请求属性，必须要设置contentType
            connection.setRequestProperty("accept", "*/*");
            connection.setRequestProperty("Content-Type","application/json");
            connection.setRequestMethod("PUT");
            connection.setRequestProperty("user-agent",
                    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            // 建立实际的连接
            connection.connect();
            //写入body数据，catchNode
            OutputStreamWriter out = new OutputStreamWriter(connection.getOutputStream(), "UTF-8"); // utf-8编码

            out.append(answerNode.getCatchNodeJson());
            out.flush();
            out.close();

            BufferedReader in = new BufferedReader(new InputStreamReader(
                    connection.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                logger.info(line);
                result.append(line);
            }
            return result.toString();

        } catch (IOException e) {
            logger.info(e.getMessage());
            logger.info(e.toString());
            e.printStackTrace();
        }
        return null;
    }

    public  void test(){
        String result = "";
        String uploadTime=String.valueOf(System.currentTimeMillis());
        System.out.println(uploadTime);
        String source="CrowdReview";
        String url="http://www.mooctest.net/api/common/generalUploadCaughtNode/"+"2664"+"/"+uploadTime+"/"+"298714"+"/"+source;
        try{
            URL realUrl = new URL(url);
            // 打开和URL之间的连接
            HttpURLConnection connection;
            connection = (HttpURLConnection)realUrl.openConnection();
            connection.setDoInput(true);
            connection.setDoOutput(true);
            // 设置请求属性，必须要设置contentType
            connection.setRequestProperty("accept", "*/*");
            connection.setRequestProperty("Content-Type","application/json");
            connection.setRequestMethod("PUT");
            connection.setRequestProperty("user-agent",
                    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            // 建立实际的连接
            connection.connect();
            //写入body数据，catchNode
            OutputStreamWriter out = new OutputStreamWriter(connection.getOutputStream(), "UTF-8"); // utf-8编码
            out.append("[{\n" +
                    "\t\"nodeName\": \"5e3932aa4cedfd000d4e371b/5e3932aa4cedfd000d4e3716/5e3932aa4cedfd000d4e3717/0\",\n" +
                    "\t\"ifCatch\": false,\n" +
                    "\t\"category\": \"CrowdReview\"\n" +
                    "}, {\n" +
                    "\t\"nodeName\": \"5e3932aa4cedfd000d4e371b/5e3932aa4cedfd000d4e3716/5e3932aa4cedfd000d4e3717/1\",\n" +
                    "\t\"ifCatch\": true,\n" +
                    "\t\"category\": \"CrowdReview\"\n" +
                    "}, {\n" +
                    "\t\"nodeName\": \"5e3932aa4cedfd000d4e371b/5e3932aa4cedfd000d4e3716/5e3932aa4cedfd000d4e3718\",\n" +
                    "\t\"ifCatch\": true,\n" +
                    "\t\"category\": \"CrowdReview\"\n" +
                    "}, {\n" +
                    "\t\"nodeName\": \"5e3932aa4cedfd000d4e371b/5e3932aa4cedfd000d4e3716/5e3932aa4cedfd000d4e3719\",\n" +
                    "\t\"ifCatch\": false,\n" +
                    "\t\"category\": \"CrowdReview\"\n" +
                    "}]");
            out.flush();
            out.close();

            BufferedReader in = new BufferedReader(new InputStreamReader(
                    connection.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
            System.out.println(result);

        } catch (IOException e) {
            e.printStackTrace();
        }


    }



}
