package edu.nju.service;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpSession;

import org.json.JSONArray;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import edu.nju.dao.BugDao;
import edu.nju.dao.BugMirrorDao;
import edu.nju.dao.BugPageDao;
import edu.nju.dao.KWDao;
import edu.nju.entities.Bug;
import edu.nju.entities.BugMirror;
import edu.nju.entities.BugPage;
//import edu.nju.util.Algorithm;
//import edu.nju.util.Algorithm_1;
import edu.nju.util.StringMatch;

@Service
public class RecommendService {
	
	@Autowired
	BugMirrorDao mirrordao;
	
	@Autowired
	BugDao bugdao;
	
	@Autowired
	BugPageDao pagedao;
	
	@Autowired
	KWDao kwdao;
	
	public List<BugMirror> getList (String case_take_id, String report_id) {
//		Algorithm algorithm = new Algorithm_1();
		List<BugMirror> results = mirrordao.findByCase(case_take_id, report_id);
//		return algorithm.sort(results);
		return results;
	}
	
	public List<String> getListIds (String case_take_id) {
		List<BugMirror> results = mirrordao.findByCase(case_take_id);
		List<String> lists = new ArrayList<String>();
		for(BugMirror mirror: results) {
			lists.add(mirror.getId());
		}
		return lists;
	}
	
	public Bug getDetail (String id) {
		return bugdao.findByid(id);
	}
	
	public BugMirror getMirror (String id) {
		return mirrordao.findById(id);
	}
	
	public String getTitle (String id) {
		return bugdao.findByid(id).getTitle();
	}
	
	public String getReport(String id) {
		return bugdao.findByid(id).getReport_id();
	}
	
	//根据bug的本质三属性进行推荐
//	@SuppressWarnings("unchecked")
//	public Map<BugMirror, Float> recommend (String case_take_id, String type, String content, boolean flag, HttpSession session) {
//		
//		List<BugMirror> results = new ArrayList<BugMirror>();
//		
//		if(session.getAttribute("rec") == null) {
//			results = findByNothing(case_take_id, type, content, (String)session.getAttribute("report"));
//			Map<String, String> map = new LinkedHashMap<String, String>();
//			map.put(type, content);
//			session.setAttribute("path", map);
//		} else {
//			Map<String, String> map = (Map<String, String>)session.getAttribute("path");
//			if(map == null) { map = new LinkedHashMap<String, String>(); }
//			if(map.containsKey(type)) {
//				return pathExist(case_take_id, type, content, session, map);
//			}
//			map.put(type, content);
//			session.setAttribute("path", map);
//			results = findByNow(type, content, (List<BugMirror>) session.getAttribute("rec"));
//		}
//		
//		session.setAttribute("rec", results);
//		if(flag) {
//			Algorithm algorithm = new Algorithm_1();
//			if(results != null) {results = algorithm.sort(results);}
//			if(session.getAttribute("title") != null) {return recommandByTitle((String)session.getAttribute("title"), session);}
//			if(session.getAttribute("des") != null) {return recommandByDes((String)session.getAttribute("des"), session);}
//		}
//		
//		Map<String, String> map = (Map<String, String>)session.getAttribute("path");
//		return finalScore(results, map.size());
//	}
	
	//根据bug的本质三属性进行推荐
	@SuppressWarnings("unchecked")
	public Map<BugMirror, Float> recommend (String case_take_id, String type, String content, boolean flag, HttpSession session) {
		
		List<BugMirror> results = new ArrayList<BugMirror>();
		
		if(session.getAttribute("rec") == null) {
			Map<String, String> map = new LinkedHashMap<String, String>();
			map.put(type, content);
			session.setAttribute("path", map);
			results = getList(case_take_id, "test");
		} else {
			Map<String, String> map = (Map<String, String>)session.getAttribute("path");
			if(map == null) { map = new LinkedHashMap<String, String>(); }
			if(map.containsKey(type)) {
				return pathExist(case_take_id, type, content, session, map);
			}
			map.put(type, content);
			session.setAttribute("path", map);
			results = (List<BugMirror>) session.getAttribute("rec");
		}
		session.setAttribute("rec", results);
		if(flag) {
			if(session.getAttribute("title") != null) {return recommandByTitle((String)session.getAttribute("title"), session);}
			if(session.getAttribute("des") != null) {return recommandByDes((String)session.getAttribute("des"), session);}
		}
		
		Map<String, String> map = (Map<String, String>)session.getAttribute("path");
		return finalScore(results, map);
	}
	
	//根据页面进行推荐
	@SuppressWarnings("unchecked")
	public Map<BugMirror, Float> recommndByPage(String case_take_id, String type, String content, boolean flag, HttpSession session){
		
		List<BugPage> results = new ArrayList<BugPage>();
		List<BugMirror> mirrors = new ArrayList<BugMirror>();
		
		if(session.getAttribute("page") == null) {
			results = findPages(case_take_id, type, content);
			mirrors = findMirror(getIds(results), (String)session.getAttribute("report"));
			Map<String, String> map = new LinkedHashMap<String, String>();
			map.put(type, content);
			session.setAttribute("path", map);
		} else {
			Map<String, String> map = (Map<String, String>)session.getAttribute("path");
			if(map.containsKey(type)) {
				return pathExist(case_take_id, type, content, session, map);
			} else {
				map.put(type, content);
				session.setAttribute("path", map);
				results = findByPages(type, content, (List<BugPage>) session.getAttribute("page"));
				mirrors = findByMirror(getIds(results), (List<BugMirror>) session.getAttribute("rec"));
			}
		}
		
		session.setAttribute("page", results);
		session.setAttribute("rec", mirrors);
		
		if(flag) {
//			Algorithm algorithm = new Algorithm_1();
//			if(mirrors != null) {mirrors = algorithm.sort(mirrors);}
			if(session.getAttribute("title") != null) {return recommandByTitle((String)session.getAttribute("title"), session);}
			if(session.getAttribute("des") != null) {return recommandByDes((String)session.getAttribute("des"), session);}
		}
		
		Map<String, String> map = (Map<String, String>)session.getAttribute("path");
//		for(Entry<String, String> entry : map.entrySet()) {
//			System.out.println(entry.getKey());
//			System.out.println(entry.getValue());
////			String content = ;
//		}

//		System.out.println("mirror");
//		for(BugMirror mirror : mirrors) {
//			System.out.println(mirror.getId());
//		}
		return finalScore(mirrors, map);
	}
	
	//根据Title进行文本匹配
	@SuppressWarnings("unchecked")
	public Map<BugMirror, Float> recommandByTitle(String content, HttpSession session){
		session.setAttribute("title", content);
		if(session.getAttribute("rec") == null) {
			List<BugMirror> temp = mirrordao.findByCase((String)session.getAttribute("case"), (String)session.getAttribute("report"));
			if(temp != null) {session.setAttribute("rec", temp);}
		}
		if(session.getAttribute("rec") != null) {return count(content, (List<BugMirror>)session.getAttribute("rec"), true, session);}
		return null;
	}
	
	//根据description进行文本匹配
	@SuppressWarnings("unchecked")
	public Map<BugMirror, Float> recommandByDes(String content, HttpSession session){
		session.setAttribute("des", content);
		if(session.getAttribute("rec") == null) {
			List<BugMirror> temp = mirrordao.findByCase((String)session.getAttribute("case"), (String)session.getAttribute("report"));
			if(temp != null) {session.setAttribute("rec", temp);}
		}
		if(session.getAttribute("rec") != null) {return count(content, (List<BugMirror>)session.getAttribute("rec"), false, session);}
		return null;
	}
	
	//从缓存中获取
//	private List<BugMirror> findByNow(String type, String content, List<BugMirror> lists){
//		List<BugMirror> results = new ArrayList<BugMirror>();
//		switch(type) {
//			case "bug_category":
//				for(BugMirror mirror : lists) {
//					if(mirror.getBug_category().equals(content)) {results.add(mirror);}
//				}
//				break;
//			case "severity":
//				for(BugMirror mirror : lists) {
//					if(mirror.getSeverity() == severityTranse(content)) {results.add(mirror);}
//				}
//				break;
//			case "recurrent":
//				for(BugMirror mirror : lists) {
//					if(mirror.getRecurrent() == recurrentTranse(content)) {results.add(mirror);}
//				}
//				break;
//		}
//		return results;
//	}

	//从数据库中获取
//	private List<BugMirror> findByNothing(String case_take_id, String type, String content, String report_id){
//		switch(type) {
//			case "bug_category":
//				return mirrordao.findByCategory(case_take_id, content, report_id);
//			case "severity":
//				return mirrordao.findBySeverity(case_take_id, severityTranse(content), report_id);
//			case "recurrent":
//				return mirrordao.findByRecurrent(case_take_id, recurrentTranse(content), report_id);
//		}
//		return null;
//	}
	
	//从数据库中查找页面
	private List<BugPage> findPages(String case_take_id, String type, String content){
		switch(type) {
			case "page1":
				return pagedao.findByPage1(case_take_id, content);
			case "page2":
				return pagedao.findByPage2(case_take_id, content);
			case "page3":
				return pagedao.findByPage3(case_take_id, content);
		}
		return null;
	}
	
	//从缓存中查找页面
	private List<BugPage> findByPages(String type, String content, List<BugPage> lists){
		List<BugPage> results = new ArrayList<BugPage>();
		switch(type) {
			case "page1":
				for(BugPage page : lists) {
					if(page.getPage1().equals(content)) {results.add(page);}
				}
				break;
			case "page2":
				for(BugPage page : lists) {
					if(page.getPage2().equals(content)) {results.add(page);}
				}
				break;
			case "page3":
				for(BugPage page : lists) {
					if(page.getPage3().equals(content)) {results.add(page);}
				}
				break;
		}
		return results;
	}
	
	//从数据库中获取指定id的bug
	private List<BugMirror> findMirror(List<String> ids, String report_id){
		List<BugMirror> results = new ArrayList<BugMirror>();
		results = mirrordao.findByIds(ids, report_id);
		return results;
	}
	
	//从缓存中进行筛选符合条件的
	private List<BugMirror> findByMirror(List<String> ids, List<BugMirror> lists){
		List<BugMirror> results = new ArrayList<BugMirror>();
		for(BugMirror mirror : lists) {
			if(ids.contains(mirror.getId())) {results.add(mirror);}
		}
		return results;
	}
	
	//根据页面获取bug_id
	private List<String> getIds(List<BugPage> pages) {
		List<String> ids = new ArrayList<String>();
		for(int i = 0; i < pages.size(); i ++) {
			ids.add(pages.get(i).getId());
		}
		return ids;
	}
	
	//该选项已经选择过
	private Map<BugMirror, Float> pathExist(String case_take_id, String type, String content, HttpSession session, Map<String, String> map){
		session.removeAttribute("rec");
		session.removeAttribute("path");
		session.removeAttribute("page");
		Map<BugMirror, Float> results = new LinkedHashMap<BugMirror, Float>();
		for(Entry<String, String> entry: map.entrySet()) {
			if(entry.getKey().equals(type)) { continue; }
			if(entry.getKey().contains("page")) {
				recommndByPage(case_take_id, entry.getKey(), entry.getValue(), false, session);
			} else {
				recommend(case_take_id, entry.getKey(), entry.getValue(), false, session);
			}
		}
		if(type.contains("page")) { results = recommndByPage(case_take_id, type, content, true, session); }
		else { results = recommend(case_take_id, type, content, true, session); }
		return results;
	}


	@SuppressWarnings("unchecked")
	private Map<BugMirror, Float> count(String content, List<BugMirror> lists, boolean type, HttpSession session) {
		StringMatch match = new StringMatch();
		Map<BugMirror, Float> tmap = new HashMap<BugMirror, Float>();
		Map<BugMirror, Float> result = new LinkedHashMap<BugMirror, Float>();
		Map<String, BugMirror> bmap = new HashMap<String, BugMirror>();
		Map<String, String> titleMap = new HashMap<String, String>();
		Map<String, String> desMap = new HashMap<String, String>();
		Map<String, String> pmap = null;

		//文本相似度计算，主要是使用StringMatch
		//获取keywords中已经分词完毕的关键词
		for(BugMirror mirror: lists) {
			String id = mirror.getId();
			bmap.put(id, mirror);
			titleMap.put(id, kwdao.findById(id).getTitle());
			desMap.put(id, kwdao.findById(id).getDescription());
		}
		//对于关键词和用户输入的信息进行文本相似度计算
		for(BugMirror mirror: lists) {
			String id = mirror.getId();
			float score = 0;
			//recommendByTitle:type==true
			//recommendByDes:type==false
			if(type) {
				//ansj算法进行中文分词
				//wmd算法计算相似度？
				score += match.score(match.Ansj(content), match.Ansj(titleMap.get(id))) * 30;
				if(session.getAttribute("des") != null) {
					score += match.score(match.Ansj((String)session.getAttribute("des")), match.Ansj(desMap.get(id))) * 40;
				}
			} else {
				score += match.score(match.Ansj(content), match.Ansj(desMap.get(id))) * 40;
				if(session.getAttribute("title") != null) {
					score += match.score(match.Ansj((String)session.getAttribute("title")), match.Ansj(titleMap.get(id))) * 30;
				}
			}
			tmap.put(mirror, score);
		}
		List<Entry<BugMirror, Float>> tlist = new ArrayList<Entry<BugMirror, Float>>(tmap.entrySet());
		//根据score排序
		Collections.sort(tlist, (a, b) -> (Float.compare(b.getValue(), a.getValue())));
		if(session.getAttribute("path") != null) {
			pmap = (Map<String, String>)session.getAttribute("path");
		}

		//只推荐前六个
		for(int i = 0; i < tlist.size() && i < 6; i ++) {
			float score = (float) (tlist.get(i).getValue());
			result.put(tlist.get(i).getKey(), score + categoryCount(tlist.get(i).getKey(), pmap));
		}
		tlist = new ArrayList<Entry<BugMirror, Float>>(result.entrySet());
		Collections.sort(tlist, (a, b) -> (Float.compare(b.getValue(), a.getValue())));
		result.clear();
		for(Entry<BugMirror, Float> entry: tlist) {
			result.put(entry.getKey(), entry.getValue());
		}
		return result;
	}
	
	private Map<BugMirror, Float> finalScore(List<BugMirror> lists, Map<String, String> map) {
		Map<BugMirror, Float> rmap = new LinkedHashMap<BugMirror, Float>();
		for(BugMirror mirror : lists) {
			rmap.put(mirror, categoryCount(mirror, map));
		}
		return rmap;
	}
	
	private float categoryCount(BugMirror mirror, Map<String, String> map) {
		if(map == null) { return 0;}
		int score = 0;
		int size = map.size();
		for(Entry<String, String> entry : map.entrySet()) {
			String type = entry.getKey();
			String content = entry.getValue();
			switch(type) {
				case "bug_category":
					size --;
					if(content.equals(mirror.getBug_category())) { score ++; }
					break;
				case "severity":
					size --;
					if(severityTranse(content) == mirror.getSeverity()) { score ++; }
					break;
				case "recurrent":
					size --;
					if(recurrentTranse(content) == mirror.getRecurrent()) { score ++; }
					break;
			}		
		}
		return (float) ((score + size) * 5);
	}
	
	private int severityTranse(String str) {
		if(str.equals("待定")) {
			return 1;
		}
		if(str.equals("较轻")) {
			return 2;
		}
		if(str.equals("一般")) {
			return 3;
		}
		if(str.equals("严重")) {
			return 4;
		}
		if(str.equals("紧急")) {
			return 5;
		}
		return 0;
	}
	
	private int recurrentTranse(String str) {
		if(str.equals("其他")) {
			return 1;
		}
		if(str.equals("无规律复现")) {
			return 2;
		}
		if(str.equals("小概率复现")) {
			return 3;
		}
		if(str.equals("大概率复现")) {
			return 4;
		}
		if(str.equals("必现")) {
			return 5;
		}
		return 0;
	}

	public void createSimilarBugLog(HttpSession session,List<BugMirror> mirrorList,List<Float> scores){
		Bug userBug=constructUserBug(session);
		StringBuilder log=new StringBuilder("userJson:");
		log.append(objectToStr(userBug,','));
		log.append("\n");
		log.append("similarJson:");
//
//		JSONObject similarJson=new JSONObject();
//		similarJson.put("userJson",new JSONObject(userBug));
//		JSONArray similarBugJSONArray=new JSONArray();

		//如果传过来的列表是未排过序的，就需要自己排序
//		Map<BugMirror,Float> hashMap=new HashMap<>();
//		for(int i=0;i<mirrorList.size();i++){
////			System.out.println(scores.get(i));
//			hashMap.put(mirrorList.get(i),scores.get(i));
//		}
//		//根据score排序
//		List<Entry<BugMirror, Float>> tList = new ArrayList<Entry<BugMirror, Float>>(hashMap.entrySet());
//		Collections.sort(tList,(a,b)->(Float.compare(b.getValue(),a.getValue())));
//
//		for(Entry<BugMirror, Float> entry:tList){
//			Bug similarBug=bugdao.findByid(entry.getKey().getId());
//			JSONObject similarBugJSONObject=new JSONObject();
//			similarBugJSONObject.put("bugDetail",new JSONObject(similarBug));
//			similarBugJSONObject.put("score",entry.getValue());
//			System.out.println(entry.getValue());
//			similarBugJSONArray.put(similarBugJSONObject);
//		}

		for(int i=0;i<mirrorList.size();i++){
			Bug similarBug=bugdao.findByid(mirrorList.get(i).getId());
			log.append(objectToStr(similarBug,','));
			log.append("score:"+scores.get(i)+",");
			log.append("\n");
//			JSONObject similarBugJSONObject=new JSONObject();
//			similarBugJSONObject.put("bugDetail",new JSONObject(similarBug));
//			similarBugJSONObject.put("score",scores.get(i));
//			similarBugJSONArray.put(similarBugJSONObject);
		}

//		similarJson.put("similarJson",similarBugJSONArray);

		Logger logger= LoggerFactory.getLogger(RecommendService.class);
		logger.info(String.valueOf(log));
	}


	private static StringBuilder objectToStr(Object o,char separator){
		StringBuilder sb=new StringBuilder();
		Field[] fields=o.getClass().getDeclaredFields();
		String[] fieldNames=new String[fields.length];
		for(int i=0;i<fields.length;i++){
//            System.out.println(fields[i].getType());
			fieldNames[i]=fields[i].getName();
			sb.append(fieldNames[i]+":");
			sb.append(getFieldValueByName(fieldNames[i],o));
			sb.append(separator);
		}
		return sb;
	}
	private static Object getFieldValueByName(String fieldName, Object o) {
		try {
			String firstLetter = fieldName.substring(0, 1).toUpperCase();
			String getter = "get" + firstLetter + fieldName.substring(1);
			Method method = o.getClass().getMethod(getter, new Class[] {});
			Object value = method.invoke(o, new Object[] {});
			return value;
		} catch (Exception e) {

			return null;
		}
	}

	private Bug constructUserBug( HttpSession session){
		String case_take_id="";
		String create_time_millis=Long.toString(System.currentTimeMillis());
		String bug_category="";
		String description="";
		String img_url="";
		int severity=0;
		int recurrent=0;
		String title="";
		String report_id="";
		String bug_page="";
		String case_id="";
		if(session.getAttribute("case")!=null){
			case_take_id=(String)session.getAttribute("case");
			case_id=case_take_id.split("-")[0];
		}
		if(session.getAttribute("path")!=null){
			LinkedHashMap<String,String> path=(LinkedHashMap<String,String>)session.getAttribute("path");
			if(path.containsKey("page1")){
				bug_page+=path.get("page1");
				if(path.containsKey("page2")){
					bug_page+=("-"+path.get("page2"));
					if(path.containsKey("page3")){
						bug_page+=("-"+path.get("page3"));
					}
				}
			}
			if(path.containsKey("bug_category")){
				bug_category=path.get("bug_category");
			}
			if(path.containsKey("severity")){
				severity=severityTranse(path.get("severity"));
			}
			if(path.containsKey("recurrent")){
				recurrent=recurrentTranse(path.get("recurrent"));
			}

		}
		if(session.getAttribute("title")!=null){
			title=(String)session.getAttribute("title");
		}
		if(session.getAttribute("des")!=null){
			description=(String)session.getAttribute("des");
		}
		Bug userBug=new Bug(case_take_id,create_time_millis,bug_category,description,img_url,severity,recurrent,title,report_id,bug_page,case_id);
//		JSONObject userBugJson=new JSONObject(userBug);
//		System.out.println(userBugJson);
		return userBug;

	}
	
}