package edu.nju.service;

import edu.nju.dao.*;
import edu.nju.entities.*;
import edu.nju.model.ReviewWorkerVO;
import edu.nju.util.OssAliyun;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.util.CellRangeAddress;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileOutputStream;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class ReviewAnalyzeService {

    @Autowired
    ReviewAnswerDao reviewAnswerDao;
    @Autowired
    ReviewItemDao reviewItemDao;
    @Autowired
    ReviewWorkerDao reviewWorkerDao;
    @Autowired
    ReviewJobDao reviewJobDao;
    @Autowired
    ReviewPaperDao reviewPaperDao;
    @Autowired
    ReviewReportDao reviewReportDao;


    public JSONObject analyzeItem(String itemId,String reportId,String jobId){
        JSONObject jsonObject=new JSONObject();
        ReviewItem reviewItem=reviewItemDao.findById(itemId);
        String type=reviewItem.getType();
        List<ReviewAnswer>reviewAnswers=reviewAnswerDao.getItemReportJobAnswers(itemId,reportId,jobId);
        if(type.equals("File")){
            Map<String,List<String>>map=new HashMap<>();
            for(int i=0;i<reviewAnswers.size();i++){
                ReviewAnswer reviewAnswer=reviewAnswers.get(i);
                List<String>urls=reviewAnswer.getFile_url();
                if(urls.size()!=0){
                    String workerId=reviewAnswer.getWorker_id();
                    ReviewWorker worker=reviewWorkerDao.find(workerId);
                    ReviewWorkerVO workerVO=reviewWorkerDao.getEduInfo(worker);
                    map.put(new JSONObject(workerVO).toString(),urls);
                }
            }
            jsonObject=new JSONObject(map);
        }
        else if(type.equals("Description")){
            Map<String,String>map=new HashMap<>();
            for(int i=0;i<reviewAnswers.size();i++){
                ReviewAnswer reviewAnswer=reviewAnswers.get(i);
                List<String>answers=reviewAnswer.getAnswers();
                if(answers.size()!=0){
                    String answer=answers.get(0);
                    String workerId=reviewAnswer.getWorker_id();
                    ReviewWorker worker=reviewWorkerDao.find(workerId);
                    ReviewWorkerVO workerVO=reviewWorkerDao.getEduInfo(worker);
                    map.put(new JSONObject(workerVO).toString(),answer);
                }
            }
            jsonObject=new JSONObject(map);
        }
        else{
            Map<Integer,Integer>map=new HashMap<>();
            for(int i=0;i<reviewAnswers.size();i++){
                ReviewAnswer reviewAnswer=reviewAnswers.get(i);
                List<String>answers=reviewAnswer.getAnswers();
                for(int j=0;j<answers.size();j++){
                    if(map.containsKey(j)){
                        if(answers.get(j).equals("1")){
                            map.put(j,map.get(j)+1);
                        }
                    }else{
                        if(answers.get(j).equals("1")){
                            map.put(j,1);
                        }else {
                            map.put(j, 0);
                        }
                    }
                }
            }
            jsonObject=new JSONObject(map);
        }
        return jsonObject;
    }



    public String getExcel (String jobId){
        int rIndex = 0;
        int iIndex = 0;
        int oIndex = 0;
        int tempRI;
        int temp  ;
        try {
            HSSFWorkbook wb = new HSSFWorkbook();
            tempRI = 0;
            FileOutputStream fileOut = null; // 为了输出excel 文件
            ReviewJob job = reviewJobDao.findJob(jobId);
            HSSFSheet sheet1 = wb.createSheet(job.getName());
            List<ReviewReport> reports = reviewReportDao.getReportsByPaperId(job.getPaper_id());
            Row rTemp ;
            Cell cTemp;
            DecimalFormat df = new DecimalFormat("#.00");
            for( ReviewReport r : reports){
                List<ReviewItem> items = reviewItemDao.findItemsByReport(r.getId());
                tempRI = 0;
                for (ReviewItem i : items){
                    List<ReviewAnswer> answers = reviewAnswerDao.getItemReportJobAnswers(i.getId(),r.getId(),jobId);
                    int [] answerCount = new int [answers.size()];
                    StringBuffer sb = new StringBuffer() ;
                    switch (i.getType()){
                        case "Single" :
                        case "Multiple":
                            for(ReviewAnswer answer : answers){
                                for( int ii = 0 ;ii < answer.getAnswers().size();ii++){
                                    answerCount[ii] ++;
                                }
                            }
                            for( int ii =0;ii<i.getOptions().size();ii++){
                                rTemp = sheet1.createRow(oIndex++);
                                cTemp = rTemp.createCell(2);
                                cTemp.setCellValue(i.getOptions().get(ii)+":"+answerCount[ii]+" - "+df.format(100.0*answerCount[ii]/ (Arrays.stream(answerCount).sum()==0?1:Arrays.stream(answerCount).sum()))+"%");
                            }
                            break;
                        case "Description" :
                            for(ReviewAnswer answer : answers){
                                sb.append(answer.getAnswers().toString()+"\n");
                            }
                            rTemp = sheet1.createRow(oIndex++);
                            cTemp = rTemp.createCell(2);
                            cTemp.setCellValue(sb.toString());
                            break;
                        case "File":
                            for(ReviewAnswer answer : answers){
                                sb.append(answer.getFile_url()+"\n");
                            }
                            rTemp = sheet1.createRow(oIndex++);
                            cTemp = rTemp.createCell(2);
                            cTemp.setCellValue(sb.toString());
                            break;
                        default:
                            break;
                    }
                    if(i.getOptions().size()<=1){ // File or description
                        temp = iIndex ;
                    }else{
                        temp = iIndex + i.getOptions().size()-1;
                        sheet1.addMergedRegion(new CellRangeAddress(iIndex,temp,1,1));
                    }
                    rTemp = sheet1.getRow(iIndex);
                    cTemp = rTemp.createCell(1);
                    cTemp.setCellValue(i.getDescription());
                    iIndex = temp+1;
                    if(i.getOptions().size()<=1){
                        tempRI += 1;
                    }else{
                        tempRI+=i.getOptions().size();
                    }
                }
                if(items.size()!=0){
                    // 如果没有item的情况
                    tempRI=tempRI==0?1:tempRI;
                    temp = rIndex + tempRI -1;
                    if(temp!=rIndex){
                        sheet1.addMergedRegion(new CellRangeAddress(rIndex,temp,0,0));
                    }
                    rTemp = sheet1.getRow(rIndex);
                    cTemp = rTemp.createCell(0);
                    cTemp.setCellValue(r.getName()+"-"+r.getDescription());
                    rIndex = temp+1;
                }
            }
            fileOut = new FileOutputStream("/Users/insomnialee/Desktop/temp/"+jobId+".xls");
            wb.write(fileOut);
            File file = new File("/Users/insomnialee/Desktop/temp/"+jobId+".xls");
            OssAliyun.uploadFile("auditExcel/"+jobId+".xls",file);
            return "http://mooctest-site.oss-cn-shanghai.aliyuncs.com/auditExcel/"+jobId+".xls";
        }catch (Exception e){
            e.printStackTrace();
            return null;
        }
    }
}
